import streamlit as st
from email.mime.text import MIMEText
from email.mime.multipart import MIMEMultipart
from email.message import EmailMessage
import yaml
import bcrypt
import random
import smtplib
import os
from dotenv import find_dotenv,load_dotenv

dotenv_path = find_dotenv()
load_dotenv(dotenv_path)

SENDER_EMAIL = os.getenv('SENDER_EMAIL')
SENDER_PASSWORD = os.getenv('SENDER_PASSWORD')


from dependencies import *
from generate_otp import *
from send_otp import *


if "load_state" not in st.session_state:
    st.session_state.load_state = False

def sign_up():
    st.header("Sign Up")
    name = st.text_input("Name")
    email1 = str(st.text_input("Email Address"))
    password = st.text_input("Password", type="password")
    confirm_password = st.text_input("Confirm Password", type="password")
    if password != confirm_password:
        st.error("Passwords do not match.")
        return
    otp = generate_otp()
    entered_otp = st.text_input("Enter OTP")
    if st.button("Send OTP") :
        send_otp_to_email(email1, otp) 
    if st.button("Sign Up"):
        st.experimental_memo.clear()
        if entered_otp == otp:
            users = {}
            try:
                with open("users.yaml", "r") as file:
                    users = yaml.load(file, Loader=yaml.FullLoader)
            except:
                pass
            if email1 in users:
                st.error("User already exists.")
            else:
                users[email1] = {"name": name, "password": password}
                with open("users.yaml", "w") as file:
                    yaml.dump(users, file)
                st.success("User signed up successfully.")

        else:
            st.error("Invalid OTP.")


from dependencies import *

def send_otp_to_email(email, OTP):
    em = EmailMessage()
    sender_email = SENDER_EMAIL
    sender_password = SENDER_PASSWORD
    receiver_email = email
    em["From"] = sender_email
    em["To"] = receiver_email
    em["Subject"] = "OTP Verification"
    em.set_content(MIMEText(f"Your OTP is {OTP}"))
    with smtplib.SMTP_SSL("smtp.gmail.com", 465) as smtp:
        smtp.login(sender_email, sender_password)
        smtp.sendmail(sender_email, receiver_email, em.as_string())


from setuptools import setup, find_packages
import codecs
import os


VERSION = '1.0.0'
DESCRIPTION = 'This package allows you to create an authntication system in streamlit'
LONG_DESCRIPTION = 'Streamlit Authentication System With Sign-UP,Login,Reset PAssword and OTP Verification Functions'

# Setting up
setup(
    name="streamlit-otp-auth",
    version=VERSION,
    author="Anjishnu SAw",
    author_email="anjishnusaw91@gmail.com",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    packages=find_packages(),
    install_requires=['streamlit', 'random', 'yaml','bcrypt','smtplib','dotenv'],
    keywords=['python', 'authentication', 'signup-login', 'otp', 'secure auth', 'streamlit authenticator'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)


from dependencies import *
from generate_otp import *
from send_otp import *
from reset_password import *

if "load_state" not in st.session_state:
    st.session_state.load_state = False

def log_in():
    st.header("Log In")
    email = st.text_input("Email Address")
    password = st.text_input("Password", type="password")
    if st.button("Log In"):
        users = {}
        try:
            with open("users.yaml", "r") as file:
                users = yaml.load(file, Loader=yaml.FullLoader)
        except:
            pass
        if email not in users:
            st.error("User does not exist.")
        elif users[email]["password"] != password:
            st.error("Invalid password.")
        else:
            st.success("Logged in successfully.")
    else:
        reset_password()



from dependencies import *
from signup import *
from login import *

if "load_state" not in st.session_state:
    st.session_state.load_state = False

def main():
    st.set_page_config(page_title="Authentication System", page_icon="🔒", layout="wide")
    st.title("Authentication System")
    hide_st_style = """
                <style>
                #MainMenu {visibility: hidden;}
                footer {visibility: hidden;}
                footer:after{
                    content:'        Copyright @ 2022: Anjishnu Saw (anjishnu@anjishnusaw.tk)';
                    display:bloack;
                    visibility: visible;
                    position:relative;
                    color:yellow
                }
                header {visibility: hidden;}
                </style>
                """
    st.markdown(hide_st_style, unsafe_allow_html=True)
    menu = ["Sign Up", "Log In"]
    choice = st.sidebar.selectbox("Select an option", menu)

    if choice == "Sign Up":
        sign_up()
    elif choice == "Log In":
        log_in()


if __name__ == "__main__":
    main()



from dependencies import *
from generate_otp import *
from send_otp import *

if "load_state" not in st.session_state:
    st.session_state.load_state = False

def reset_password():
    if st.button("Forgot password") or st.session_state.load_state:
        st.session_state.load_state = True
        email = str(st.text_input("Email"))
        otp_password = st.text_input("Enter your Otp set to email")
        reset_otp = generate_otp()
        if st.button("Send OTP") :
            send_otp_to_email(email, reset_otp)
            st.write("Your password reset otp has benn sent to your registered email address")
        if otp_password == reset_otp:
            new_password = st.text_input("Enter your new password : ")
            new_password_confirm = st.text_input("Confirm your new password : ")
            if st.button("Reset password") :
                users = {}
                try:
                    with open("users.yaml",'r') as file :
                        users = yaml.load(file,Loader = yaml.FullLoader)
                except:
                    pass
                if new_password_confirm == new_password and email in users:
                    st.experimental_memo.clear()
                    users[email]["password"] = new_password
                    with open("users.yaml", "w") as file:
                        yaml.dump(users, file)
                    st.success("Password Reset Suucessful")
                elif new_password != new_password_confirm:
                    st.error("Passwords do not match")
                else:
                    st.error("User registration not found. Please check your entered address or signup")
            elif otp_password != reset_otp:
                st.write("Please enter correct OTP")